/******************************************************************************

ES_Bonofaf
-------------------------------------------------------------------------------
Small, plant-eating mouse-like creature

-------------------------------------------------------------------------------
Author:     Jochen Werner
Changed:    Jul 7, 2002
Version:    1.0

******************************************************************************/

package EcoSystem;

import javax.media.j3d.*;
import javax.vecmath.*;
import java.util.*;


public class ES_Bononaf extends ES_Animal
{
  final static int STATE_WANDER           = 0;
  final static int STATE_ENROUTETOPLACE   = 1;
  final static int STATE_SEARCHFOOD       = 2;
  final static int STATE_ENROUTETOFOOD    = 3;
  final static int STATE_EATFOOD          = 4;
  final static int STATE_GATHERFOOD       = 5;
  final static int STATE_ENROUTETOHIVE    = 6;
  final static int STATE_PLACEFOODATHIVE  = 7;
  final static int STATE_SLEEPATHIVE      = 8;
  final static int STATE_ENROUTETOPARTNER = 9;
  final static int STATE_WAITFORPARTNER   = 10;
  final static int STATE_ATPARTNER        = 11;
  final static int STATE_BEARCHILDS       = 12;
  final static int STATE_FLEEING          = 13;

  final static int BREED_NOT              = 0;
  final static int BREED_INHEAT           = 1;
  final static int BREED_PREGNANT         = 2;
  final static int BREED_BREEDING         = 3;

  final static int SEX_MALE               = 0;
  final static int SEX_FEMALE             = 1;

  final static float DECIDE_EAT           = 15.0f;
  final static float DECIDE_SLEEP         = 45.0f;

  final static float CRITICAL_EAT         = 50.0f;
  final static float CRITICAL_SLEEP       = 165.0f;

  final static int MAX_BONONAFS           = 40;

  //                                        1 minute * 5 = 5 minutes +- some time
  final long         CRITICAL_AGE         = Math.round(((60*1000)*5)*ES_Util.GetRndFloat(0.9f,1.2f));
  final long         PREGNANCYTIME        = 1000*30;

  final static float  MOVESPEED           = 0.3f;
  final static float  LOOKSPEED           = 90.0f;
  final static double LOOKANGLE           = Math.PI / 8;
  final static double LOOKDISTANCE        = 3.0f;

  //                                         20 secs
  final static long   HIVEMOVE_INTERVAL     = 20*1000;
  final static float  HIVEMOVE_MINDISTANCE  = 1.5f;

  final static long   WAITFORPARTNERTIME  = 10*1000;

  final protected long  timeborn          = ES_Util.GetESTime();
  final protected int   mySex             = (int)ES_Util.GetRndLong(SEX_MALE,SEX_FEMALE);
  protected float       tiredness         = 0.0f;
  protected float       carryfood         = 0.0f;
  protected float       hunger            = 0.0f;

  protected long        timeasked         = 0;
  protected long        timegotpregnant   = 0;

  protected Transform3D   lastKnownFoodPos  = null;

  protected ES_LifeForm wantToEat     = null;
  protected ES_Desolok  myHunter      = null;
  protected ES_Bononaf  partner       = null;
  protected ES_BononafHive myHive     = null;
  protected ES_BononafCorpse myCorpse = null;
  protected long nextHiveMoveTime   = ES_Util.GetESTime()+1000;

  protected int breedState        = 0;
  protected int curState          = 0;

  public ES_Bononaf(Transform3D t, ES_Window w)
  {
    super(t,w);
    myFlags.set(ES.Bononaf);
    myFlags.set((mySex == SEX_MALE ? ES.SEX_Male : ES.SEX_Female));

    TransformGroup myNewBody = myWindow.myModelCache.getModel("Bononaf"+(mySex == SEX_MALE ? "Male" : "Female"));

    myBody.insertChild(myNewBody,ES.BNI_Body);

    setMyName(ES_Util.GetRandomName());

    homeBranch.compile();
    myWindow.objRoot.addChild(homeBranch);

    myHive = new ES_BononafHive(this,t,w);

    myWindow.numBononafs++;

  }

  public void FrameThink()
  {
    super.FrameThink();

    if (myMoves.QueueIsEmpty())
    {
      if      (curState == STATE_WANDER)
      {
        if (ES_Util.GetRndFloat() > 0.5f)
          myMoves.EnqueueMove(new ES_Move(this,myMoves,MOVESPEED, LOOKSPEED,new Point3f(ES_Util.GetRndFloat(-1.0f,1.0f) , 0, ES_Util.GetRndFloat(-1.0f,1.0f) )));
        else
          myMoves.EnqueueMove(new ES_Move(this,myMoves,MOVESPEED, LOOKSPEED,new Point3f(ES_Util.GetRndFloat(-0.5f,0.5f) , 0, ES_Util.GetRndFloat(0.6f,1.2f) )));

        myMoves.SetMoveMode(ES.MM_MoveASAP);
        setFlag(ES.isMoving);
        return;
      }
      else if (curState == STATE_ENROUTETOPLACE)
      {
        curState = STATE_WANDER;
        return;
      }
      else if (curState == STATE_ENROUTETOFOOD)
      {
        curState = STATE_EATFOOD;
        return;
      }
      else if (curState == STATE_ENROUTETOHIVE)
      {
        if (breedState == BREED_PREGNANT && ES_Util.GetESTime() > timegotpregnant+PREGNANCYTIME)
        {
          curState = STATE_BEARCHILDS;
          return;
        }
        if (carryfood > 0.0f)
        {
          curState = STATE_PLACEFOODATHIVE;
          return;
        }
        if (tiredness > DECIDE_SLEEP)
        {
          curState = STATE_SLEEPATHIVE;
          return;
        }
        curState = STATE_WANDER;
        return;

      }
      else if (curState == STATE_WAITFORPARTNER)
      {
        if (ES_Util.GetESTime() > timeasked + WAITFORPARTNERTIME)
        {
          partner.TellMale();
          partner = null;
          curState = STATE_WANDER;
          return;
        }
      }
      else if (curState == STATE_ENROUTETOPARTNER)
      {
        curState = STATE_ATPARTNER;
        partner.TellFemale();
        return;
      }
      else if (curState == STATE_FLEEING)
      {
        if (myHunter == null)
        {
          curState = STATE_WANDER;
          return;
        }
        if (!myHunter.homeBranch.isLive())
        {
          myHunter = null;
          curState = STATE_WANDER;
          return;
        }
        if (ES_Util.CanSeeLifeFormInCone(myWindow.objRoot,GetTransform(),LOOKDISTANCE,LOOKANGLE,myHunter))
        {
          final Vector3f huntervec = new Vector3f();
          myHunter.GetTransform().get(huntervec);
          final Point3f  hunterloc = new Point3f((Tuple3f)huntervec);
          ES_Util.MakeLocalCoordinates(this,hunterloc);
          hunterloc.negate();
          huntervec.set((Tuple3f)hunterloc);
          huntervec.normalize();
          huntervec.x += ES_Util.GetRndFloat(-1.0f,1.0f);
          huntervec.scale(ES_Util.GetRndFloat(0.3f,1.0f));
          hunterloc.set((Tuple3f)huntervec);
          myMoves.EnqueueMove(new ES_Move(this,myMoves,MOVESPEED*2.0f, LOOKSPEED*1.5f,hunterloc));
          myMoves.SetMoveMode(ES.MM_MoveASAP);
          setFlag(ES.isMoving);
          return;
        }
        else
        {
          curState = STATE_WANDER;
          return;
        }
      }

    }

  }

  public void Think()
  {
    super.Think();

    myThink.setNextThink((int)ES_Util.GetRndLong(900,1100));
//    setMyName("H="+hunger+", T=" + tiredness+"->S="+curState);

    CheckCriticals();

    if (curState == STATE_SLEEPATHIVE)
    {
      Sleep();
      return;
    }
    if (curState == STATE_EATFOOD)
    {
      Eat();
      return;
    }
    if (curState == STATE_ATPARTNER)
    {
      Fuck();
      return;
    }
    if (curState == STATE_PLACEFOODATHIVE)
    {
      PlaceFoodAtHive();
      return;
    }
    if (curState == STATE_BEARCHILDS)
    {
      BearChilds();
      return;
    }

    Tire();

    Hunger();

    Breed();

    if (curState == STATE_WANDER && ES_Util.GetESTime() > nextHiveMoveTime && ES_Util.GetRndLong(0,10)<1)
    {
      checkHiveMove();
    }
  }

  public void Hunger()
  {
    hunger+=ES_Util.GetRndFloat()+0.2f;

    if (hunger > DECIDE_EAT && curState == STATE_WANDER)
    {
        SearchFood();
    }
  }

  public void Tire()
  {
    tiredness+=ES_Util.GetRndFloat()+0.2f;

    if (tiredness > DECIDE_SLEEP && curState == STATE_WANDER)
    {
      GoToHive();
    }
  }

  public void Breed()
  {
    if (mySex == SEX_MALE)
    {
      if (curState != STATE_WANDER)
        return;

      if (ES_Util.GetRndFloat()>0.4)
        return;

      if (partner != null)
        return;

      partner = (ES_Bononaf)ES_Util.GetClosestLifeFormInCone(myWindow.objRoot,GetTransform(),LOOKDISTANCE*1.75d,LOOKANGLE,ES.Bononaf,ES.SEX_Female,ES.SEX_InHeat);

      if (partner != null)
      {

        if (!partner.AskFemale(this))
        {
          partner = null;
          return;
        }
       ES_Util.BuildRoute(this,partner.GetTransform(),0.12f,MOVESPEED*2.5f,LOOKSPEED*2.0f);
       curState = STATE_ENROUTETOPARTNER;
       breedState = BREED_INHEAT;
      }

    }
    else
    {
      if (breedState == BREED_NOT)
      {
        if (ES_Util.GetRndFloat()>0.1)
          return;

        breedState = BREED_INHEAT;
        setFlag(ES.SEX_InHeat);

      }
      else if (breedState == BREED_INHEAT)
      {

      }
      else if (breedState == BREED_PREGNANT)
      {
        if (ES_Util.GetESTime() > timegotpregnant+PREGNANCYTIME && curState == STATE_WANDER)
        {
          GoToHive();
        }
        if (curState == STATE_WANDER)
        {
          if (myHive.myFood < 5.0f)
          {
            SearchFood();
          }
        }
      }
      else if (breedState == BREED_BREEDING)
      {
        if (curState == STATE_WANDER)
        {
          if (myHive.myFood < 7.5f)
          {
            SearchFood();
          }
        }
      }
    }
  }


  public boolean AskFemale(ES_Bononaf lover)
  {
    if (breedState != BREED_INHEAT)
      return false;

    if (curState != STATE_WANDER)
      return false;

    if (partner != null)
      return false;

    partner = lover;
    myMoves.AbortCurMove();
    myMoves.ClearQueue();
    curState = STATE_WAITFORPARTNER;
    timeasked = ES_Util.GetESTime();
    return true;

  }

  public void TellFemale()
  {
    curState = STATE_ATPARTNER;
  }

  public void TellMale()
  {
    partner = null;
    curState = STATE_WANDER;
    breedState = BREED_NOT;
  }

  public void Fuck()
  {
    partner  = null;
    curState = STATE_WANDER;
    breedState = BREED_NOT;

    if (mySex == SEX_FEMALE)
    {
      clearFlag(ES.SEX_InHeat);

      if (ES_Util.GetRndFloat()>0.8f)
        return;

      breedState = BREED_PREGNANT;
      timegotpregnant = ES_Util.GetESTime();
    }

  }

  public void BearChilds()
  {
    breedState = BREED_BREEDING;
    myHive.BearChilds();

    if (carryfood > 0.0f)
      curState = STATE_PLACEFOODATHIVE;
    else if (tiredness > DECIDE_SLEEP)
      curState = STATE_SLEEPATHIVE;
    else
      curState = STATE_WANDER;
  }

  public void GoToHive()
  {
    ES_Util.BuildRoute(this,myHive.GetTransform(),0.0f,MOVESPEED*2.0f,LOOKSPEED);
    curState = STATE_ENROUTETOHIVE;

  }

  public void Sleep()
  {
    if (tiredness > 0.0f)
    {
      tiredness -= 4.0f;
    }
    else
    {
      curState = STATE_WANDER;
    }
  }

  public void SearchFood()
  {
    wantToEat = ES_Util.GetClosestLifeFormInCone(myWindow.objRoot,GetTransform(),LOOKDISTANCE,LOOKANGLE,ES.Flower);

    if (wantToEat == null && hunger > CRITICAL_EAT - 25.0f && lastKnownFoodPos != null)
    {
       ES_Util.BuildRoute(this,new Transform3D(lastKnownFoodPos),0.12f,MOVESPEED*2.0f,LOOKSPEED);
       lastKnownFoodPos = null;
       curState = STATE_ENROUTETOPLACE;
       return;
    }

    if (wantToEat == null && hunger > CRITICAL_EAT - 15.0f)
        wantToEat = ES_Util.GetClosestLifeFormInCone(myWindow.objRoot,GetTransform(),LOOKDISTANCE*1.25d,LOOKANGLE,ES.BononafCorpse);

    if (wantToEat != null)
    {
       ES_Util.BuildRoute(this,wantToEat.GetTransform(),0.12f,MOVESPEED*2.0f,LOOKSPEED);
       curState = STATE_ENROUTETOFOOD;
    }
  }

  public void PlaceFoodAtHive()
  {
    myHive.AddFood(carryfood);
    carryfood = 0.0f;
    if (tiredness > DECIDE_SLEEP)
      curState = STATE_SLEEPATHIVE;
    else
      curState = STATE_WANDER;
  }

  public void Eat()
  {
    if (wantToEat == null)
    {
      curState = STATE_WANDER;
      return;
    }

    if (!wantToEat.homeBranch.isLive())
    {
      curState = STATE_WANDER;
      return;
    }


    float lesshunger = -1.0f;
    if (wantToEat.hasFlag(ES.Flower))
    {
      lastKnownFoodPos = new Transform3D(wantToEat.GetTransform());

      float flowergrowth = ((ES_Flower)wantToEat).growth;

      final float oldgrowth = flowergrowth;
      flowergrowth -= hunger;
      lesshunger    = Math.min(hunger,oldgrowth);
      hunger       -= lesshunger;

      ((ES_Flower)wantToEat).setGrowth(flowergrowth);
    }
    else if (wantToEat.hasFlag(ES.BononafCorpse))
    {
      hunger -= 30.0f;
      lesshunger = 30.0f;
      wantToEat.homeBranch.detach();
    }

    if (lesshunger > 0.0f)
    {
      if (breedState == BREED_PREGNANT)
      {
        carryfood += lesshunger * 0.25f;
        hunger +=    lesshunger * 0.25f;
        if (CheckCarryFood())
          return;
      }
      else if (breedState == BREED_BREEDING)
      {
        carryfood += lesshunger * 0.4f;
        hunger +=    lesshunger * 0.4f;
        if (CheckCarryFood())
          return;
      }
    }
    curState = STATE_WANDER;
  }

  public boolean CheckCarryFood()
  {
    if (carryfood > 5.0f)
    {
      GoToHive();
      return true;
    }
    return false;
  }

  public void CheckCriticals()
  {
    if (tiredness > CRITICAL_SLEEP)
      Die();

    if (hunger > CRITICAL_EAT)
      Die();

    if ( ((System.currentTimeMillis() - ES_Util.startTime) - timeborn) > CRITICAL_AGE)
      Die();
  }

  public void Die()
  {
    myCorpse = new ES_BononafCorpse(getMyName(), mySex, GetTransform(),myWindow);
    homeBranch.detach();
    myHive.homeBranch.detach();
    myWindow.numBononafs--;
  }

  public void checkHiveMove()
  {
    if (breedState < BREED_PREGNANT)
    {

      final Point3f  hivepoint = new Point3f();
      final Point3f  nowlocpoint = new Point3f();

      myHive.GetTransform().transform(hivepoint);
      GetTransform().transform(nowlocpoint);
      if (hivepoint.distance(nowlocpoint) > HIVEMOVE_MINDISTANCE )
      {
        final Vector flarhive = ES_Util.GetLifeFormsInCyl(myWindow.objRoot,myHive.GetTransform(),1.0d,ES.Flower);
        final Vector flarnowloc = ES_Util.GetLifeFormsInCyl(myWindow.objRoot,GetTransform(),1.0d,ES.Flower);

        if (flarhive.size()+1 < flarnowloc.size())
        {
          myHive.homeBranch.detach();
          myHive = new ES_BononafHive(this,GetTransform(),myWindow);
        }
      }
    }

    nextHiveMoveTime = ES_Util.GetESTime() + HIVEMOVE_INTERVAL;

  }

  public void Hunt_GiveUp()
  {
    myHunter = null;
    curState = STATE_WANDER;
  }

  public ES_BononafCorpse Hunt_Success()
  {
    Die();
    return myCorpse;
  }

  public void Hunt_Start(ES_Desolok hunter)
  {
    myHunter = hunter;
    myMoves.AbortCurMove();
    myMoves.ClearQueue();
    curState = STATE_FLEEING;
  }

  public String curStateToString()
  {
    final StringBuffer retBuf = new StringBuffer();

    if (curState == STATE_WANDER)
      retBuf.append("Wandering");
    else if (curState == STATE_ENROUTETOPLACE)
      retBuf.append("En Route To Place");
    else if (curState == STATE_SEARCHFOOD)
      retBuf.append("Searching Food");
    else if (curState == STATE_ENROUTETOFOOD)
      retBuf.append("En Route To Food");
    else if (curState == STATE_EATFOOD)
      retBuf.append("Eating Food");
    else if (curState == STATE_GATHERFOOD)
      retBuf.append("Gathering Food");
    else if (curState == STATE_ENROUTETOHIVE)
      retBuf.append("En Route To Hive");
    else if (curState == STATE_PLACEFOODATHIVE)
      retBuf.append("Placing Food At Hive");
    else if (curState == STATE_SLEEPATHIVE)
      retBuf.append("Sleeping");
    else if (curState == STATE_ENROUTETOPARTNER)
      retBuf.append("En Route To Sex Partner");
    else if (curState == STATE_WAITFORPARTNER)
      retBuf.append("Waiting For Sex Partner");
    else if (curState == STATE_ATPARTNER)
      retBuf.append("Having Sex");
    else if (curState == STATE_BEARCHILDS)
      retBuf.append("Bearing Childs");
    else if (curState == STATE_FLEEING)
      retBuf.append("Fleeing");

    return new String(retBuf);

  }

  public String curBreedStateToString()
  {

    final StringBuffer retBuf = new StringBuffer();

    if (breedState == BREED_NOT)
      retBuf.append("Not Breeding");
    else if (breedState == BREED_INHEAT)
      retBuf.append("In Heat");
    else if (breedState == BREED_PREGNANT)
      retBuf.append("Pregnant");
    else if (breedState == BREED_BREEDING)
      retBuf.append("Breeding");

    return new String(retBuf);
  }

}