/******************************************************************************

ES_Cloud
-------------------------------------------------------------------------------
One of the Random-generated Clouds that move over the EcoSystem
They rain from time to time and hence give flowers water to grow.
-------------------------------------------------------------------------------
Author:     Jochen Werner
Changed:    Jul 12, 2002
Version:    1.0

******************************************************************************/

package EcoSystem;

import javax.media.j3d.*;
import javax.vecmath.*;
import java.util.*;


public class ES_Cloud extends ES_Animal
{
  final protected int BNI_RAIN = 2;

  final protected BranchGroup RainBranch = new BranchGroup();

  protected boolean isRaining = false;

  public ES_Cloud(Transform3D t, ES_Window w)
  {
    super(t,w);

    myFlags.set(ES.Cloud);

    final TransformGroup myNewBody = myWindow.myModelCache.getModel("Cloud");
    final TransformGroup myRain    = myWindow.myModelCache.getModel("Rain");

    ES_Util.SetESCapabilities(RainBranch);

    RainBranch.addChild(myRain);

    myBody.insertChild(myNewBody,ES.BNI_Body);

    setMyName("Cloud");

    homeBranch.compile();
    myWindow.objRoot.addChild(homeBranch);

    myThink.setNextThink(1000);
  }

  public void FrameThink()
  {
    super.FrameThink();

    if (myMoves.QueueIsEmpty())
    {
      final ES_Move newMove = new ES_Move(this,myMoves,0.1f, 270.0f,new Point3f(ES_Util.GetRndFloat(-0.1f,0.1f) , 0, ES_Util.GetRndFloat(1.0f,1.5f) ));

      if (!newMove.isMoveValid())
      {
        homeBranch.detach();
      }

      myMoves.EnqueueMove(newMove);
      myMoves.SetMoveMode(ES.MM_MoveASAP);
      setFlag(ES.isMoving);


    }
  }

  public void Think()
  {
    super.Think();

    if (ES_Util.GetRndFloat() > 0.8)
    {
      isRaining = !isRaining;


      if (isRaining)
      {
        myBody.insertChild(RainBranch,BNI_RAIN);
      }
      else
      {
        myBody.removeChild(BNI_RAIN);
      }
    }

  }

}