/******************************************************************************

ES_Desolok
-------------------------------------------------------------------------------
mid-sized creature that hunts Bononafs.

-------------------------------------------------------------------------------
Author:     Jochen Werner
Changed:    Jul 15, 2002
Version:    1.0

******************************************************************************/

package EcoSystem;

import javax.media.j3d.*;
import javax.vecmath.*;
import java.util.*;


public class ES_Desolok extends ES_Animal
{
  final static int STATE_WANDER           = 0;
  final static int STATE_ENROUTETOPLACE   = 1;
  final static int STATE_SEARCHFOOD       = 2;
  final static int STATE_ENROUTETOFOOD    = 3;
  final static int STATE_EATFOOD          = 4;
  final static int STATE_GATHERFOOD       = 5;
  final static int STATE_ENROUTETOHIVE    = 6;
  final static int STATE_PLACEFOODATHIVE  = 7;
  final static int STATE_SLEEPATHIVE      = 8;
  final static int STATE_ENROUTETOPARTNER = 9;
  final static int STATE_WAITFORPARTNER   = 10;
  final static int STATE_ATPARTNER        = 11;
  final static int STATE_BEARCHILDS       = 12;
  final static int STATE_HUNTING          = 13;

  final static int BREED_NOT              = 0;
  final static int BREED_INHEAT           = 1;
  final static int BREED_PREGNANT         = 2;
  final static int BREED_BREEDING         = 3;

  final static int SEX_MALE               = 0;
  final static int SEX_FEMALE             = 1;

  final static float DECIDE_EAT           = 20.0f;
  final static float DECIDE_SLEEP         = 60.0f;

  final static float CRITICAL_EAT         = 90.0f;
  final static float CRITICAL_SLEEP       = 160.0f;

  final static int MAX_DESOLOKS           = 15;

  //                                        1 minute * 8 = 8 minutes +- some time
  final long         CRITICAL_AGE         = Math.round(((60*1000)*5)*ES_Util.GetRndFloat(0.9f,1.2f));
  final long         PREGNANCYTIME        = 1000*60;

  final static float  MOVESPEED           = 0.4f;
  final static float  LOOKSPEED           = 50.0f;
  final static double LOOKANGLE           = Math.PI / 10;
  final static double LOOKDISTANCE        = 2.5f;

  //                                         40 secs
  final static long   HIVEMOVE_INTERVAL     = 40*1000;
  final static float  HIVEMOVE_MINDISTANCE  = 1.5f;

  final static long   WAITFORPARTNERTIME  = 10*1000;

  final protected long  timeborn          = ES_Util.GetESTime();
  final protected int   mySex             = (int)ES_Util.GetRndLong(SEX_MALE,SEX_FEMALE);
  protected float       tiredness         = 0.0f;
  protected float       carryfood         = 0.0f;
  protected float       hunger            = 0.0f;

  protected long        timeasked         = 0;
  protected long        timegotpregnant   = 0;

  protected Transform3D   lastKnownFoodPos  = null;

  protected ES_LifeForm wantToEat   = null;
  protected ES_Desolok  partner     = null;
  protected ES_DesolokHive myHive   = null;
  protected long nextHiveMoveTime   = ES_Util.GetESTime()+1000;
  protected ES_Bononaf  myPrey      = null;

  protected int breedState        = 0;
  protected int curState          = 0;

  public ES_Desolok(Transform3D t, ES_Window w)
  {
    super(t,w);
    myFlags.set(ES.Desolok);
    myFlags.set((mySex == SEX_MALE ? ES.SEX_Male : ES.SEX_Female));

    TransformGroup myNewBody = myWindow.myModelCache.getModel("Desolok"+(mySex == SEX_MALE ? "Male" : "Female"));

    myBody.insertChild(myNewBody,ES.BNI_Body);

    setMyName(ES_Util.GetRandomName());

    homeBranch.compile();
    myWindow.objRoot.addChild(homeBranch);

    myHive = new ES_DesolokHive(this,t,w);

    myWindow.numDesoloks++;

  }

  public void FrameThink()
  {
    super.FrameThink();

    if (myMoves.QueueIsEmpty())
    {
      if      (curState == STATE_WANDER)
      {
        if (ES_Util.GetRndFloat() > 0.25f)
          myMoves.EnqueueMove(new ES_Move(this,myMoves,MOVESPEED, LOOKSPEED,new Point3f(ES_Util.GetRndFloat(-0.5f,0.5f) , 0, ES_Util.GetRndFloat(-0.5f,0.5f) )));
        else
          myMoves.EnqueueMove(new ES_Move(this,myMoves,MOVESPEED, LOOKSPEED,new Point3f(ES_Util.GetRndFloat(-0.25f,0.25f) , 0, ES_Util.GetRndFloat(0.3f,0.6f) )));

        myMoves.SetMoveMode(ES.MM_MoveASAP);
        setFlag(ES.isMoving);
        return;
      }
      else if (curState == STATE_ENROUTETOPLACE)
      {
        curState = STATE_WANDER;
        return;
      }
      else if (curState == STATE_ENROUTETOFOOD)
      {
        curState = STATE_EATFOOD;
        return;
      }
      else if (curState == STATE_ENROUTETOHIVE)
      {
        if (breedState == BREED_PREGNANT && ES_Util.GetESTime() > timegotpregnant+PREGNANCYTIME)
        {
          curState = STATE_BEARCHILDS;
          return;
        }
        if (carryfood > 0.0f)
        {
          curState = STATE_PLACEFOODATHIVE;
          return;
        }
        if (tiredness > DECIDE_SLEEP)
        {
          curState = STATE_SLEEPATHIVE;
          return;
        }
        curState = STATE_WANDER;
        return;

      }
      else if (curState == STATE_WAITFORPARTNER)
      {
        if (ES_Util.GetESTime() > timeasked + WAITFORPARTNERTIME)
        {
          partner.TellMale();
          partner = null;
          curState = STATE_WANDER;
          return;
        }
      }
      else if (curState == STATE_ENROUTETOPARTNER)
      {
        curState = STATE_ATPARTNER;
        partner.TellFemale();
        return;
      }
      else if (curState == STATE_HUNTING)
      {
        if (myPrey == null)
        {
          curState = STATE_WANDER;
          return;
        }
        if (!myPrey.homeBranch.isLive())
        {
          myPrey = null;
          curState = STATE_WANDER;
          return;
        }

        final Vector3f myvec = new Vector3f();
        GetTransform().get(myvec);
        final Point3f myloc  = new Point3f((Tuple3f)myvec);
        myPrey.GetTransform().get(myvec);
        final Point3f preyloc  = new Point3f((Tuple3f)myvec);

        if (myloc.distance(preyloc) < 0.24f)
        {
          Hunt_Success();
          return;
        }

        ES_Util.BuildRoute(this,myPrey.GetTransform(),-0.1f,MOVESPEED*2.0f,LOOKSPEED*1.25f);
        myMoves.SetMoveMode(ES.MM_MoveASAP);
        setFlag(ES.isMoving);

        return;
      }

    }

  }

  public void Think()
  {
    super.Think();
    myThink.setNextThink((int)ES_Util.GetRndLong(900,1100));

    CheckCriticals();

    if (curState == STATE_HUNTING)
    {
      Hunt_Start(myPrey);
      return;
    }

    if (curState == STATE_SLEEPATHIVE)
    {
      Sleep();
      return;
    }
    if (curState == STATE_EATFOOD)
    {
      Eat();
      return;
    }
    if (curState == STATE_ATPARTNER)
    {
      Fuck();
      return;
    }
    if (curState == STATE_PLACEFOODATHIVE)
    {
      PlaceFoodAtHive();
      return;
    }
    if (curState == STATE_BEARCHILDS)
    {
      BearChilds();
      return;
    }

    Tire();

    Hunger();

    Breed();

    if (curState == STATE_WANDER && ES_Util.GetESTime() > nextHiveMoveTime && ES_Util.GetRndLong(0,10)<1)
    {
      checkHiveMove();
    }
  }

  public void Hunger()
  {
    hunger+=ES_Util.GetRndFloat()+0.2f;

    if (hunger > DECIDE_EAT && curState == STATE_WANDER)
    {
        SearchFood();
    }
  }

  public void Tire()
  {
    tiredness+=ES_Util.GetRndFloat()+0.2f;

    if (tiredness > DECIDE_SLEEP && curState == STATE_WANDER)
    {
      GoToHive();
    }
  }

  public void Breed()
  {
    if (mySex == SEX_MALE)
    {
      if (curState != STATE_WANDER)
        return;

      if (ES_Util.GetRndFloat()>0.4)
        return;

      if (partner != null)
        return;

      partner = (ES_Desolok)ES_Util.GetClosestLifeFormInCone(myWindow.objRoot,GetTransform(),LOOKDISTANCE*1.75d,LOOKANGLE,ES.Desolok,ES.SEX_Female,ES.SEX_InHeat);

      if (partner != null)
      {

        if (!partner.AskFemale(this))
        {
          partner = null;
          return;
        }
       ES_Util.BuildRoute(this,partner.GetTransform(),0.18f,MOVESPEED*2.5f,LOOKSPEED*2.0f);
       curState = STATE_ENROUTETOPARTNER;
       breedState = BREED_INHEAT;
      }

    }
    else
    {
      if (breedState == BREED_NOT)
      {
        if (ES_Util.GetRndFloat()>0.1)
          return;

        breedState = BREED_INHEAT;
        setFlag(ES.SEX_InHeat);

      }
      else if (breedState == BREED_INHEAT)
      {

      }
      else if (breedState == BREED_PREGNANT)
      {
        if (ES_Util.GetESTime() > timegotpregnant+PREGNANCYTIME && curState == STATE_WANDER)
        {
          GoToHive();
        }
        if (curState == STATE_WANDER)
        {
          if (myHive.myFood < 5.0f)
          {
            SearchFood();
          }
        }
      }
      else if (breedState == BREED_BREEDING)
      {
        if (curState == STATE_WANDER)
        {
          if (myHive.myFood < 7.5f)
          {
            SearchFood();
          }
        }
      }
    }
  }


  public boolean AskFemale(ES_Desolok lover)
  {
    if (breedState != BREED_INHEAT)
      return false;

    if (curState != STATE_WANDER)
      return false;

    if (partner != null)
      return false;

    partner = lover;
    myMoves.AbortCurMove();
    myMoves.ClearQueue();
    curState = STATE_WAITFORPARTNER;
    timeasked = ES_Util.GetESTime();
    return true;

  }

  public void TellFemale()
  {
    curState = STATE_ATPARTNER;
  }

  public void TellMale()
  {
    partner = null;
    curState = STATE_WANDER;
    breedState = BREED_NOT;
  }

  public void Fuck()
  {
    partner  = null;
    curState = STATE_WANDER;
    breedState = BREED_NOT;

    if (mySex == SEX_FEMALE)
    {
      clearFlag(ES.SEX_InHeat);

      if (ES_Util.GetRndFloat()>0.8f)
        return;

      breedState = BREED_PREGNANT;
      timegotpregnant = ES_Util.GetESTime();
    }

  }

  public void BearChilds()
  {
    breedState = BREED_BREEDING;
    myHive.BearChilds();

    if (carryfood > 0.0f)
      curState = STATE_PLACEFOODATHIVE;
    else if (tiredness > DECIDE_SLEEP)
      curState = STATE_SLEEPATHIVE;
    else
      curState = STATE_WANDER;
  }

  public void GoToHive()
  {
    ES_Util.BuildRoute(this,myHive.GetTransform(),0.0f,MOVESPEED*2.0f,LOOKSPEED);
    curState = STATE_ENROUTETOHIVE;

  }

  public void Sleep()
  {
    if (tiredness > 0.0f)
    {
      tiredness -= 4.0f;
    }
    else
    {
      curState = STATE_WANDER;
    }
  }

  public void SearchFood()
  {
    wantToEat = ES_Util.GetClosestLifeFormInCone(myWindow.objRoot,GetTransform(),LOOKDISTANCE,LOOKANGLE,ES.BononafCorpse);

    if (wantToEat == null)
    {
       wantToEat = ES_Util.GetClosestLifeFormInCone(myWindow.objRoot,GetTransform(),LOOKDISTANCE,LOOKANGLE,ES.Bononaf);

       if (wantToEat != null)
       {
         Hunt_Start((ES_Bononaf)wantToEat);
         return;
       }
    }

    if (wantToEat == null && hunger > CRITICAL_EAT - 25.0f && lastKnownFoodPos != null)
    {
       ES_Util.BuildRoute(this,new Transform3D(lastKnownFoodPos),0.18f,MOVESPEED*2.0f,LOOKSPEED);
       lastKnownFoodPos = null;
       curState = STATE_ENROUTETOPLACE;
       return;
    }

    if (wantToEat == null && hunger > CRITICAL_EAT - 15.0f)
        wantToEat = ES_Util.GetClosestLifeFormInCone(myWindow.objRoot,GetTransform(),LOOKDISTANCE*1.25d,LOOKANGLE,ES.DesolokCorpse);

    if (wantToEat != null)
    {
       ES_Util.BuildRoute(this,wantToEat.GetTransform(),0.18f,MOVESPEED*2.0f,LOOKSPEED);
       curState = STATE_ENROUTETOFOOD;
    }
  }

  public void PlaceFoodAtHive()
  {
    myHive.AddFood(carryfood);
    carryfood = 0.0f;
    if (tiredness > DECIDE_SLEEP)
      curState = STATE_SLEEPATHIVE;
    else
      curState = STATE_WANDER;
  }

  public void Eat()
  {
    if (wantToEat == null)
    {
      curState = STATE_WANDER;
      return;
    }

    if (!wantToEat.homeBranch.isLive())
    {
      curState = STATE_WANDER;
      return;
    }


    float lesshunger = -1.0f;
    if (wantToEat.hasFlag(ES.BononafCorpse)||wantToEat.hasFlag(ES.DesolokCorpse))
    {
      hunger -= 75.0f;
      lesshunger = 75.0f;
      wantToEat.homeBranch.detach();
    }

    if (lesshunger > 0.0f)
    {
      if (breedState == BREED_PREGNANT)
      {
        carryfood += lesshunger * 0.25f;
        hunger +=    lesshunger * 0.25f;
        if (CheckCarryFood())
          return;
      }
      else if (breedState == BREED_BREEDING)
      {
        carryfood += lesshunger * 0.4f;
        hunger +=    lesshunger * 0.4f;
        if (CheckCarryFood())
          return;
      }
    }
    curState = STATE_WANDER;
  }

  public boolean CheckCarryFood()
  {
    if (carryfood > 5.0f)
    {
      GoToHive();
      return true;
    }
    return false;
  }

  public void CheckCriticals()
  {
    if (tiredness > CRITICAL_SLEEP)
      Die();

    if (hunger > CRITICAL_EAT)
      Die();

    if ( ((System.currentTimeMillis() - ES_Util.startTime) - timeborn) > CRITICAL_AGE)
      Die();
  }

  public void Die()
  {
    ES_DesolokCorpse myCorpse = new ES_DesolokCorpse(getMyName(), mySex, GetTransform(),myWindow);
    homeBranch.detach();
    myHive.homeBranch.detach();
    myWindow.numDesoloks--;
  }

  public void checkHiveMove()
  {
    if (breedState != BREED_BREEDING)
    {

      final Point3f  hivepoint = new Point3f();
      final Point3f  nowlocpoint = new Point3f();

      myHive.GetTransform().transform(hivepoint);
      GetTransform().transform(nowlocpoint);
      if (hivepoint.distance(nowlocpoint) > HIVEMOVE_MINDISTANCE )
      {
        final Vector flarhive = ES_Util.GetLifeFormsInCyl(myWindow.objRoot,myHive.GetTransform(),1.0d,ES.Flower);
        final Vector flarnowloc = ES_Util.GetLifeFormsInCyl(myWindow.objRoot,GetTransform(),1.0d,ES.Flower);

        if (flarhive.size()+1 < flarnowloc.size())
        {
          myHive.homeBranch.detach();
          myHive = new ES_DesolokHive(this,GetTransform(),myWindow);
        }
      }
    }

    nextHiveMoveTime = ES_Util.GetESTime() + HIVEMOVE_INTERVAL;

  }

  public void Hunt_GiveUp()
  {
    myPrey.Hunt_GiveUp();
    curState = STATE_WANDER;
  }

  public void Hunt_Success()
  {
    final ES_BononafCorpse preycorpse = myPrey.Hunt_Success();
    wantToEat = preycorpse;
    ES_Util.BuildRoute(this,preycorpse.GetTransform(),0.0f,MOVESPEED,LOOKSPEED);
    curState = STATE_ENROUTETOFOOD;
    myPrey = null;
  }

  public void Hunt_Start(ES_Bononaf hunted)
  {
    curState = STATE_HUNTING;
    myPrey = hunted;
    myPrey.Hunt_Start(this);
    myMoves.AbortCurMove();
    myMoves.ClearQueue();
  }

  public String curStateToString()
  {
    final StringBuffer retBuf = new StringBuffer();

    if (curState == STATE_WANDER)
      retBuf.append("Wandering");
    else if (curState == STATE_ENROUTETOPLACE)
      retBuf.append("En Route To Place");
    else if (curState == STATE_SEARCHFOOD)
      retBuf.append("Searching Food");
    else if (curState == STATE_ENROUTETOFOOD)
      retBuf.append("En Route To Food");
    else if (curState == STATE_EATFOOD)
      retBuf.append("Eating Food");
    else if (curState == STATE_GATHERFOOD)
      retBuf.append("Gathering Food");
    else if (curState == STATE_ENROUTETOHIVE)
      retBuf.append("En Route To Hive");
    else if (curState == STATE_PLACEFOODATHIVE)
      retBuf.append("Placing Food At Hive");
    else if (curState == STATE_SLEEPATHIVE)
      retBuf.append("Sleeping");
    else if (curState == STATE_ENROUTETOPARTNER)
      retBuf.append("En Route To Sex Partner");
    else if (curState == STATE_WAITFORPARTNER)
      retBuf.append("Waiting For Sex Partner");
    else if (curState == STATE_ATPARTNER)
      retBuf.append("Having Sex");
    else if (curState == STATE_BEARCHILDS)
      retBuf.append("Bearing Childs");
    else if (curState == STATE_HUNTING)
      retBuf.append("Hunting");

    return new String(retBuf);

  }

  public String curBreedStateToString()
  {

    final StringBuffer retBuf = new StringBuffer();

    if (breedState == BREED_NOT)
      retBuf.append("Not Breeding");
    else if (breedState == BREED_INHEAT)
      retBuf.append("In Heat");
    else if (breedState == BREED_PREGNANT)
      retBuf.append("Pregnant");
    else if (breedState == BREED_BREEDING)
      retBuf.append("Breeding");

    return new String(retBuf);
  }

}