/******************************************************************************

ES_Flower
-------------------------------------------------------------------------------
small flower that grows and reproduces itself

-------------------------------------------------------------------------------
Author:     Jochen Werner
Changed:    Jul 3, 2002
Version:    1.0

******************************************************************************/

package EcoSystem;

import javax.media.j3d.*;
import javax.vecmath.*;

public class ES_Flower extends ES_Plant
{
  final static int MAX_FLOWERS       = 100;

  final static int GROW_STALK        = 1;
  final static int GROW_LEAF1        = 3;
  final static int GROW_LEAF2        = 10;
  final static int GROW_LEAF3        = 20;
  final static int GROW_BLOSSOM      = 35;
  final static int GROW_SEEDOUT      = 40;
  final static int GROW_AFTERSEEDOUT = 9;

  protected byte        timesSeeded     = 0;
  protected float       growth          = 0.0f;

  final protected double      scale         = 0.35d;
  final protected Transform3D tempTrans     = new Transform3D();
  final protected Quat4f      tempQuat      = new Quat4f();
  final protected Vector3d    tempVec       = new Vector3d();

  final BranchGroup    myStalkBranch  = new BranchGroup();
  final TransformGroup myStalk        = myWindow.myModelCache.getModel("Plant_Stalk");
  final TransformGroup myStalkTrans   = new TransformGroup();
  final BranchGroup    myLeaf1Branch  = new BranchGroup();
  final TransformGroup myLeaf1        = myWindow.myModelCache.getModel("LP_Plant_Leaf");
  final TransformGroup myLeaf1Trans   = new TransformGroup();
  final BranchGroup    myLeaf2Branch  = new BranchGroup();
  final TransformGroup myLeaf2        = myWindow.myModelCache.getModel("LP_Plant_Leaf");
  final TransformGroup myLeaf2Trans   = new TransformGroup();
  final BranchGroup    myLeaf3Branch  = new BranchGroup();
  final TransformGroup myLeaf3        = myWindow.myModelCache.getModel("LP_Plant_Leaf");
  final TransformGroup myLeaf3Trans   = new TransformGroup();
  final BranchGroup myBlossomBranch   = new BranchGroup();
  final TransformGroup myBlossom      = myWindow.myModelCache.getModel("LP_Plant_Blossom");
  final TransformGroup myBlossomTrans = new TransformGroup();

  public ES_Flower(Transform3D t, ES_Window w)
  {
    super(t,w);

    setMyName(" ");

    DisableFrameThink();

    homeBranch.compile();

    myWindow.objRoot.addChild(homeBranch);

    setFlag(ES.Flower);

    myWindow.numFlowers++;

    if (ES_Util.IsLifeFormInCyl(myWindow.objRoot,t,0.2d, ES.Flower))
    {
      Rot();
    }

    if (myWindow.numFlowers > MAX_FLOWERS)
    {
      Rot();
    }

    myFlags.set(ES.Plant);

    ES_Util.SetESCapabilities(myStalkBranch);
    ES_Util.SetESCapabilities(myStalkTrans);

    tempQuat.set(new AxisAngle4f(0,1,0,0));
    tempVec.set(0,0,0);
    tempVec.scale(scale);
    tempTrans.set(tempQuat, tempVec, scale);
    myStalkTrans.setTransform(tempTrans);
    myStalkTrans.addChild(myStalk);
    myStalkBranch.addChild(myStalkTrans);
    myStalkBranch.compile();

    ES_Util.SetESCapabilities(myLeaf1Branch);
    tempQuat.set(new AxisAngle4f(0,1,0,ES_Util.GetRndFloat(0.0f,(float)(Math.PI*2))));
    tempVec.set(0,0.1d,0);
    tempVec.scale(scale);
    tempTrans.set(tempQuat, tempVec, scale);
    ES_Util.SetESCapabilities(myLeaf1Trans);
    myLeaf1Trans.setTransform(tempTrans);
    myLeaf1Trans.addChild(myLeaf1);
    myLeaf1Branch.addChild(myLeaf1Trans);
    myLeaf1Branch.compile();

    ES_Util.SetESCapabilities(myLeaf2Branch);
    tempQuat.set(new AxisAngle4f(0,1,0,ES_Util.GetRndFloat(0.0f,(float)(Math.PI*2))));
    tempVec.set(0,0.2d,0);
    tempVec.scale(scale);
    tempTrans.set(tempQuat, tempVec, scale);
    ES_Util.SetESCapabilities(myLeaf2Trans);
    myLeaf2Trans.setTransform(tempTrans);
    myLeaf2Trans.addChild(myLeaf2);
    myLeaf2Branch.addChild(myLeaf2Trans);
    myLeaf2Branch.compile();

    ES_Util.SetESCapabilities(myLeaf3Branch);
    tempQuat.set(new AxisAngle4f(0,1,0,ES_Util.GetRndFloat(0.0f,(float)(Math.PI*2))));
    tempVec.set(0,0.3d,0);
    tempVec.scale(scale);
    tempTrans.set(tempQuat, tempVec, scale);
    ES_Util.SetESCapabilities(myLeaf3Trans);
    myLeaf3Trans.setTransform(tempTrans);
    myLeaf3Trans.addChild(myLeaf3);
    myLeaf3Branch.addChild(myLeaf3Trans);
    myLeaf3Branch.compile();

    ES_Util.SetESCapabilities(myBlossomBranch);
    tempQuat.set(new AxisAngle4f(0,1,0,0));
    tempVec.set(-0.005d,0.36d,0);
    tempVec.scale(scale);
    tempTrans.set(tempQuat, tempVec, scale);
    ES_Util.SetESCapabilities(myBlossomTrans);
    myBlossomTrans.setTransform(tempTrans);
    myBlossomTrans.addChild(myBlossom);
    myBlossomBranch.addChild(myBlossomTrans);
    myBlossomBranch.compile();

  }

  public void Think()
  {
    super.Think();

    Grow();

    if (growth <= 0)
    {
      Rot();
    }

    if (timesSeeded > 3)
    {
      Rot();
    }

    myThink.setNextThink((int)ES_Util.GetRndLong(900,1100));
  }

  protected void Grow()
  {
    setGrowth(growth+1.0f);

//    setMyName("growth:"+growth);
  }

  protected void setGrowth(float newgrowth)
  {
    if (newgrowth > GROW_SEEDOUT)
    {
      SeedOut();
      setGrowth(GROW_AFTERSEEDOUT);
      return;
    }

    if (newgrowth < GROW_BLOSSOM && growth >= GROW_BLOSSOM)
    {
      myBlossomBranch.detach();
    }
    if (newgrowth < GROW_LEAF3 && growth >= GROW_LEAF3)
    {
      myLeaf3Branch.detach();
    }
    if (newgrowth < GROW_LEAF2 && growth >= GROW_LEAF2)
    {
      myLeaf2Branch.detach();
    }
    if (newgrowth < GROW_LEAF1 && growth >= GROW_LEAF1)
    {
      myLeaf1Branch.detach();
    }
    if (newgrowth < GROW_STALK && growth >= GROW_STALK)
    {
      myStalkBranch.detach();
    }

    if (newgrowth >= GROW_STALK && growth < GROW_STALK)
    {
      myBody.addChild(myStalkBranch);
    }
    if (newgrowth >= GROW_LEAF1 && growth < GROW_LEAF1)
    {
      myBody.addChild(myLeaf1Branch);
    }
    if (newgrowth >= GROW_LEAF2 && growth < GROW_LEAF2)
    {
      myBody.addChild(myLeaf2Branch);
    }
    if (newgrowth >= GROW_LEAF3 && growth < GROW_LEAF3)
    {
      myBody.addChild(myLeaf3Branch);
    }
    if (newgrowth >= GROW_BLOSSOM && growth < GROW_BLOSSOM)
    {
      myBody.addChild(myBlossomBranch);
    }

    growth = newgrowth;

  }

  protected void Rot()
  {
    this.homeBranch.detach();
    myWindow.numFlowers--;
  }

  protected void SeedOut()
  {
//    setMyName("Seeded!");

    timesSeeded++;

    final Point3d nowloc = new Point3d(0,0,0);

    final Transform3D mytrans = GetTransform();

    mytrans.transform(nowloc);

    float tmpoffset;

    tmpoffset = ES_Util.GetRndFloat(0.21f,0.3f);
    if (ES_Util.GetRndBool())
      tmpoffset *= -1;

    nowloc.x += tmpoffset;

    tmpoffset = ES_Util.GetRndFloat(0.21f,0.3f);
    if (ES_Util.GetRndBool())
      tmpoffset *= -1;

    nowloc.z += tmpoffset;

    mytrans.setTranslation(new Vector3d((Tuple3d)nowloc));

    if (ES_Util.IsLifeFormInCyl(myWindow.objRoot,mytrans,0.2d, ES.Flower))
    {
      return;
    }

    if (myWindow.numFlowers > MAX_FLOWERS)
    {
      return;
    }

    final ES_Flower myChildFlower = new ES_Flower(mytrans, myWindow);

  }

}