/******************************************************************************

ES_LifeForm
-------------------------------------------------------------------------------
Abstract Base Class for all EcoSystem LifeForms

-------------------------------------------------------------------------------
Author:     Jochen Werner
Changed:    Feb 5, 2002
Version:    1.0

******************************************************************************/

package EcoSystem;

import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.geometry.ColorCube;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.behaviors.vp.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.util.*;

public class ES_LifeForm
{
  public    ES_BranchGroup        homeBranch    = null;

  protected ES_Window             myWindow      = null;
  protected ES_DescriptiveText    myDescription = null;
  protected TransformGroup        myDescTrans   = null;
  protected TransformGroup        myBody        = null;
  protected ES_ThinkBehavior      myThink       = null;
  protected ES_FrameThinkBehavior myFrameThink  = null;
  final protected BitSet          myFlags       = new BitSet();

  protected String                myName        = "LifeForm";

  public ES_LifeForm(Transform3D t, ES_Window w)
  {
    myFlags.set(ES.LifeForm);

    myWindow    = w;

    myBody = new TransformGroup(t);

    ES_Util.SetESCapabilities(myBody);

//    myBody.addChild(new ColorCube(0.2));

//    ES_Util.PrintCoordinateSystem(myBody,0.4f);

    if (myWindow.enableDescriptions)
    {
      myDescription = new ES_DescriptiveText(myName);
      myDescTrans = new TransformGroup();
      myDescTrans.addChild(myDescription);

      final Quat4f myDQ4f = new Quat4f();
      final Transform3D myDT = new Transform3D();

      myDT.rotX((Math.PI/2)*-1);
      myDT.get(myDQ4f);
      myDT.set(myDQ4f,new Vector3d(-0.2,0.2,0.2),1.0d);

      myDescTrans.setTransform(myDT);

      myBody.insertChild(myDescTrans,ES.BNI_Desc);
    }
    else
    {
      myBody.insertChild(null,ES.BNI_Desc);
    }

    homeBranch = new ES_BranchGroup(this);
    ES_Util.SetESCapabilities(homeBranch);
    homeBranch.addChild(myBody);

    myThink = new ES_ThinkBehavior(this,5,-1);

    homeBranch.addChild(myThink);

    myFrameThink = new ES_FrameThinkBehavior(this);

    homeBranch.addChild(myFrameThink);

  }

  public void Think()
  {

  }

  public void FrameThink()
  {

  }

  public void DisableFrameThink()
  {
    myFrameThink.setEnable(false);
    myFrameThink = null;
  }

  public void DisableThink()
  {
    myThink.setEnable(false);
    myThink = null;
  }

  protected Transform3D GetTransform()
  {
    final Transform3D tmptrans = new Transform3D();

    myBody.getTransform(tmptrans);

    return tmptrans;
  }

  //----------------------------------------
  // myName Set/Get Functions
  //----------------------------------------

  public String getMyName()
  {
    return myName;
  }

  public void setMyName(String newname)
  {
    myName = newname;

    if (myWindow.enableDescriptions)
      myDescription.setString(myName);
  }


  //----------------------------------------
  // Flag Clear Functions
  //----------------------------------------

  public void clearFlag(int flag)
  {
    myFlags.clear(flag);
  }

  public void clearFlags( int[] flags)
  {
    for(int i=0;i<flags.length;i++)
    {
      myFlags.clear(flags[i]);
    }
  }


  //----------------------------------------
  // Flag GET Functions
  //----------------------------------------

  public BitSet getFlags()
  {
    return myFlags;
  }

  //----------------------------------------
  // Flag SET Functions
  //----------------------------------------

  public void setFlag(int flag)
  {
    myFlags.set(flag);
  }

  public void setFlags(BitSet flags)
  {
    myFlags.or(flags);
  }

  public void setFlags(int[] flags)
  {
    for(int i=0;i<flags.length;i++)
    {
      myFlags.set(flags[i]);
    }
  }

  //----------------------------------------
  // Flag HAS Functions
  //----------------------------------------

  public boolean hasFlag(int flag)
  {
    return myFlags.get(flag);
  }

  public boolean hasFlag(int flag, int flag2)
  {
    return (myFlags.get(flag) && myFlags.get(flag2));
  }

  public boolean hasFlag(int flag, int flag2, int flag3)
  {
    return (myFlags.get(flag) && myFlags.get(flag2) && myFlags.get(flag3));
  }

  public boolean hasFlags( int[] flags)
  {
    for(int i=0;i<flags.length;i++)
    {
      if (!myFlags.get(flags[i]) )
        return false;
    }
    return true;
  }


}