/******************************************************************************

ES_LifeFormInfoRefreshBehavior
-------------------------------------------------------------------------------
Refreshes the LifeFormInfoWindow every n milliseconds

-------------------------------------------------------------------------------
Author:     Jochen Werner
Changed:    Jul 9, 2002
Version:    1.0

******************************************************************************/

package EcoSystem;

import javax.media.j3d.*;
import javax.vecmath.*;
import java.lang.Double;
import java.util.Enumeration;


public class ES_LifeFormInfoRefreshBehavior extends Behavior
{

  protected long cur_wakeupTime = 0;
  protected long repeats = -1;
  protected WakeupOnElapsedTime wakeupOnTime = null;
  protected ES_Window myWindow = null;

    public void initialize() {
	wakeupOn(wakeupOnTime);
    }

    public void processStimulus(Enumeration criteria)
    {
        myWindow.myInfoWindow.DoRefresh();
        myWindow.myMainFrame.setTitle("EcoSystem running for " + ((ES_Util.GetESTime() / 1000.0) > 60.0 ? ((ES_Util.GetESTime() / 1000.0 / 60.0 + " minutes.")) : (ES_Util.GetESTime() / 1000.0 + " seconds.")));

        if (repeats != 0)
        {
          if (wakeupOnTime.getElapsedFrameTime() != cur_wakeupTime)
          {
            wakeupOnTime = null;
            wakeupOnTime = new WakeupOnElapsedTime(cur_wakeupTime);
          }
          wakeupOn(wakeupOnTime);

          if ( repeats > 0 )
            repeats--;
        }
    }

    public void setNextThink( long nextthink )
    {
        cur_wakeupTime = nextthink;
    }

    public long getNextThink()
    {
      return cur_wakeupTime;
    }

    public ES_LifeFormInfoRefreshBehavior(ES_Window mywindow, long nextthink, long rep)
    {
        myWindow = mywindow;
        cur_wakeupTime = nextthink;
        repeats = rep;
        wakeupOnTime = new WakeupOnElapsedTime(nextthink);

	this.setSchedulingBounds(ES_Util.GetInfiniteBounds());
    }
}




