/******************************************************************************

ES_LifeFormInfoWindow
-------------------------------------------------------------------------------
displays informations on LifeForms in a swing window

-------------------------------------------------------------------------------
Author:     Jochen Werner
Changed:    Jul 9, 2002
Version:    1.0

******************************************************************************/


package EcoSystem;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class ES_LifeFormInfoWindow extends JFrame
{
  protected ES_Window myWindow = null;
  protected ES_LifeFormInfoRefreshBehavior myRefreshBehavior = null;
  protected ES_LifeForm myLife = null;
  protected JLabel valuelabels[];

  // Methode: public void showInfoOn(ES_LifeForm lf);
  public ES_LifeFormInfoWindow(ES_Window mywindow)
  {
    myWindow = mywindow;
    myRefreshBehavior = new ES_LifeFormInfoRefreshBehavior(myWindow,750,-1);
    myRefreshBehavior.setSchedulingBounds(ES_Util.GetInfiniteBounds());
    myWindow.objRoot.addChild(myRefreshBehavior);

    setTitle("EcoSystem LifeForm Info Window");
    setSize(350,100);
  }

  public void DoRefresh()
  {
    if (myLife == null)
      return;

    if (!this.isVisible())
      return;

    if (!myLife.homeBranch.isLive())
    {
      myLife = null;
      this.hide();
      return;
    }

    if (myLife.hasFlag(ES.Bononaf))
    {
      ES_Bononaf b = (ES_Bononaf)myLife;

      final String values[] = {"Bononaf",         b.getMyName(), (b.mySex == b.SEX_MALE ? "Male" : "Female"), Float.toString(b.hunger), Float.toString(b.tiredness), Double.toString(b.timeborn/1000), Double.toString((b.timeborn+b.CRITICAL_AGE-ES_Util.GetESTime())/1000), b.curStateToString(), b.curBreedStateToString(),Integer.toString(myWindow.numBononafs)};

      RefreshValues(values);

    }
    else if (myLife.hasFlag(ES.Desolok))
    {
      ES_Desolok d = (ES_Desolok)myLife;

      final String values[] = {"Desolok",         d.getMyName(), (d.mySex == d.SEX_MALE ? "Male" : "Female"), Float.toString(d.hunger), Float.toString(d.tiredness), Double.toString(d.timeborn/1000), Double.toString((d.timeborn+d.CRITICAL_AGE-ES_Util.GetESTime())/1000), d.curStateToString(),d.curBreedStateToString()};

      RefreshValues(values);

    }

    else if (myLife.hasFlag(ES.BononafHive))
    {
      ES_BononafHive h = (ES_BononafHive)myLife;

      final boolean f = (h.myBononaf.mySex == ES_Bononaf.SEX_FEMALE);

      final String values[] = {"Bononaf's Hive", h.myBononaf.getMyName(), (f ? "Female" : "Male"), (f ? (""+h.numChilds) : ""), ( f ? Float.toString(h.myFood) : " "), (h.numChilds >= 1 ? Float.toString(h.childGrowth[0]) : ""),(h.numChilds >= 2 ? Float.toString(h.childGrowth[1]) : ""),(h.numChilds >= 3 ? Float.toString(h.childGrowth[2]) : ""),Double.toString(Runtime.getRuntime().freeMemory()/1024.0d/1024.0d)};

      RefreshValues(values);

    }
    else if (myLife.hasFlag(ES.BononafChild))
    {
      ES_BononafChild c = (ES_BononafChild)myLife;

      final String values[] = {"Bononaf Child ", c.myMother.getMyName(), Float.toString(c.hunger), Double.toString((c.timeborn+c.CHILDTIME-ES_Util.GetESTime())/1000)};

      RefreshValues(values);

    }
    else if (myLife.hasFlag(ES.Flower))
    {
      ES_Flower f = (ES_Flower)myLife;

      final String values[] = {"Flower", Float.toString(f.growth), Integer.toString(f.timesSeeded), Integer.toString(myWindow.numFlowers) };

      RefreshValues(values);

    }

  }

  protected void RefreshValues(String values[])
  {
    for (int i=0; i<values.length;i++)
    {
      valuelabels[i].setText(values[i]);
    }
  }

  protected void GenerateLabels(String descs[], String values[])
  {
    getContentPane().removeAll();

    final GridLayout myLayout = new GridLayout(descs.length, 2, 10, 0);

    getContentPane().setLayout(myLayout);

    JLabel desclabels[]  = new JLabel[descs.length];
    valuelabels = new JLabel[values.length];

    for (int i=0; i< descs.length;i++)
    {
      desclabels[i] = new JLabel(descs[i]);
      valuelabels[i] = new JLabel(values[i]);

      getContentPane().add(desclabels[i]);
      getContentPane().add(valuelabels[i]);

    }
    Dimension winDim = myWindow.getSize();
    Dimension myDim  = this.getSize();
    Point     winLoc = myWindow.getLocationOnScreen();

    setLocation(10+winDim.width+(int)Math.round(winLoc.getX()),((int)Math.round(winLoc.getY()))-10);
    setSize(myDim.width,20*descs.length+20);
    show();

  }

  public void showInfoOn(ES_Bononaf b)
  {
    myLife = b;

    final String descs[]  = {"LifeForm is a:",  "Name:",       "Sex:",                                      "Hunger:",                "Tiredness:",                "timeborn:",                      "Lifetime remaining:",                                                 "curState:",          "breedState:",             "Total Number of Bononafs:"};
    final String values[] = {"Bononaf",         b.getMyName(), (b.mySex == b.SEX_MALE ? "Male" : "Female"), Float.toString(b.hunger), Float.toString(b.tiredness), Double.toString(b.timeborn/1000), Double.toString((b.timeborn+b.CRITICAL_AGE-ES_Util.GetESTime())/1000), b.curStateToString(), b.curBreedStateToString(), Integer.toString(myWindow.numBononafs)};

    GenerateLabels(descs,values);
  }

  public void showInfoOn(ES_Desolok d)
  {
    myLife = d;

    final String descs[]  = {"LifeForm is a:",  "Name:",       "Sex:",                                      "Hunger:",                "Tiredness:",                "timeborn:",                      "Lifetime remaining:",                                                 "curState:",          "breedState:"};
    final String values[] = {"Desolok",         d.getMyName(), (d.mySex == d.SEX_MALE ? "Male" : "Female"), Float.toString(d.hunger), Float.toString(d.tiredness), Double.toString(d.timeborn/1000), Double.toString((d.timeborn+d.CRITICAL_AGE-ES_Util.GetESTime())/1000), d.curStateToString(),d.curBreedStateToString()};

    GenerateLabels(descs,values);
  }

  public void showInfoOn(ES_BononafHive h)
  {
    myLife = h;

    final boolean f = (h.myBononaf.mySex == ES_Bononaf.SEX_FEMALE);

    final String descs[]  = {"Object is a:",   "Owning Bononaf:",       "Bononaf's Sex:",        (f ? "numChilds:" : " "),    ( f ? "Food at Hive:" : " "),          (h.numChilds >= 1 ? "childGrowth of Child 1:" : " "),      (h.numChilds >= 2 ? "childGrowth of Child 2:" : " "),      (h.numChilds >= 3 ? "childGrowth of Child 3:" : " "), "Free Memory in the JVM:", "Memory Allocated:"};
    final String values[] = {"Bononaf's Hive", h.myBononaf.getMyName(), (f ? "Female" : "Male"), (f ? (""+h.numChilds) : ""), ( f ? Float.toString(h.myFood) : " "), (h.numChilds >= 1 ? Float.toString(h.childGrowth[0]) : ""),(h.numChilds >= 2 ? Float.toString(h.childGrowth[1]) : ""),(h.numChilds >= 3 ? Float.toString(h.childGrowth[2]) : ""),Double.toString(Runtime.getRuntime().freeMemory()/1024/1024),Double.toString(Runtime.getRuntime().totalMemory()/1024/1024)};

    GenerateLabels(descs,values);
  }

  public void showInfoOn(ES_BononafChild c)
  {
    myLife = c;

    final String descs[]  = {"LifeForm is a:", "Mother:",               "Hunger:",               "ChildTime remaining:"};
    final String values[] = {"Bononaf Child ", c.myMother.getMyName(), Float.toString(c.hunger), Double.toString((c.timeborn+c.CHILDTIME-ES_Util.GetESTime())/1000)};

    GenerateLabels(descs,values);
  }

  public void showInfoOn(ES_Flower f)
  {
    myLife = f;

    final String descs[]  = {"LifeForm is a:", "Growth:",                "timesSeeded:",                  "Total Number of Flowers:" };
    final String values[] = {"Flower",         Float.toString(f.growth), Integer.toString(f.timesSeeded), Integer.toString(myWindow.numFlowers) };

    GenerateLabels(descs,values);

  }


}
