/******************************************************************************

ES_ModelCache
-------------------------------------------------------------------------------
Caches every Model used and returns the model as part of a
TransfromGroup with the right scale factor
-------------------------------------------------------------------------------
Author:     Jochen Werner
Changed:    Jul 5, 2002
Version:    1.0

******************************************************************************/

package EcoSystem;

import javax.media.j3d.*;
import java.util.Hashtable;
import com.sun.j3d.loaders.*;
import java.io.File;



public class ES_ModelCache
{
  final protected Hashtable myModels    = new Hashtable();

  final protected double scale          = 0.015d;
  final protected Loader loader         = new MS3DLoader(MS3DLoader.LOAD_ALL);

  public ES_ModelCache()
  {
  }

  public void addModel(String ms3dfilename, String key)
  {
    final java.io.File file = new java.io.File(ms3dfilename);

    if (!file.exists())
    {
       System.out.println("File not found: " + ms3dfilename);
       return;
    }

   try
   {
     if (file.getParent().length() > 0) // figure out the base path
       loader.setBasePath("");

     final Scene scene = loader.load(file.getName());
     final BranchGroup group = scene.getSceneGroup();
     myModels.put(key,group);
   }
   catch (Exception e)
   {
     e.printStackTrace();
   }

  }

  public TransformGroup getModel(String key)
  {

    final BranchGroup modelBranch = (BranchGroup)myModels.get(key);

    if (modelBranch == null)
    {
      System.out.println("Model not Precached: " + key);
      return null;
    }
    else
    {
      final Transform3D t = new Transform3D();
      t.setScale(scale);
      final TransformGroup retTrans = new TransformGroup(t);

      ES_Util.SetESCapabilities(retTrans);
      retTrans.addChild(modelBranch.cloneTree(true));

      return retTrans;
    }

  }

}

