/******************************************************************************

ES_MoveQueue
-------------------------------------------------------------------------------
Enqueues several moves and commences the moves one after another.

-------------------------------------------------------------------------------
Author:     Jochen Werner
Changed:    Feb 11, 2002
Version:    1.0

******************************************************************************/

package EcoSystem;

import javax.media.j3d.*;
import javax.vecmath.*;
import java.util.*;


public class ES_MoveQueue
{
  protected ES_Animal    myAnimal     = null;
  protected ES_Move      curMove      = null;
  final protected Vector myMoves      = new Vector();
  protected int          myMoveMode   = 0;
  protected boolean      myTrigger    = false;


  public ES_MoveQueue(ES_Animal animal)
  {
    myAnimal = animal;
    myMoveMode = 0;

  }

  public ES_MoveQueue(ES_Animal animal, int movemode)
  {
    myAnimal = animal;
    myMoveMode = movemode;

  }

  public void SetMoveMode(int movemode)
  {
    myMoveMode = movemode;
  }

  public int GetMoveMode()
  {
    return myMoveMode;
  }

  public void TriggerMove()
  {
    if((myMoveMode != ES.MM_OnTrigger) && (myMoveMode != ES.MM_OnTriggerSingle))
      return;
    myTrigger = true;

  }

  public void UnTriggerMove()
  {
    myTrigger = false;
  }

  protected void printMoves()
  {

  }

  public void performMove()
  {
    if (myMoveMode == ES.MM_DontMove)
    {
      myAnimal.clearFlag(ES.isMoving);
      return;
    }

    if ((myMoveMode == ES.MM_OnTrigger) || (myMoveMode == ES.MM_OnTriggerSingle))
    {
      if (!myTrigger)
      {
        myAnimal.clearFlag(ES.isMoving);
        return;
      }
    }

    if (myMoves.isEmpty())
    {
      myAnimal.clearFlag(ES.isMoving);
      return;
    }

    curMove = (ES_Move)myMoves.firstElement();

    if (curMove == null)
    {
      return;
    }

    if (!curMove.isMoving())
    {
      if (curMove.isMoveFinished())
      {
        if (myMoveMode == ES.MM_OnTriggerSingle)
        {
          myTrigger = false;
          myAnimal.clearFlag(ES.isMoving);
        }

        if (myMoves.size() == 1)
        {
          myMoves.clear();
          myAnimal.clearFlag(ES.isMoving);
          return;
        }
        else
        {
          myMoves.removeElement(myMoves.firstElement());
          return;
        }

      }
      else
      {
//        ES_Util.DebugPrintLn("starting move @ ");
        if (curMove.isMoveValid())
          curMove.startMove();
        else
          curMove.abortMove();
      }
    }
    else
    {
      curMove.performMove();
    }

  }

  public void AbortCurMove()
  {
     if (curMove != null)
       curMove.abortMove();
  }

  public void EnqueueMove(ES_Move move)
  {
    myMoves.addElement(move);
  }

  public void RemoveMove(ES_Move move)
  {
    myMoves.removeElement(move);
  }

  public void ClearQueue()
  {
    myMoves.clear();
  }

  public int GetQueueSize()
  {
    return myMoves.size();
  }

  public boolean QueueIsEmpty()
  {
    return myMoves.isEmpty();
  }

}