/******************************************************************************

ES_PickMouseBehavior
-------------------------------------------------------------------------------
Gets the clicked LifeForm (if any) and display its properties in the
ES_ViewLFProperties JFrame
-------------------------------------------------------------------------------
Author:     Jochen Werner
Changed:    Jul 9, 2002
Version:    1.0

******************************************************************************/

package EcoSystem;


import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.picking.*;

public class ES_PickMouseBehavior extends Behavior {

  PickCanvas pickCanvas;
  PickResult pickResult;
  ES_Window myWindow = null;

  public ES_PickMouseBehavior(Canvas3D canvas3D, BranchGroup branchGroup, ES_Window mywindow)
  {
    myWindow = mywindow;
    pickCanvas = new PickCanvas(canvas3D, branchGroup);
    pickCanvas.setTolerance(5.0f);
    pickCanvas.setMode(PickCanvas.GEOMETRY);
  }

  public void initialize() {
    wakeupOn (new WakeupOnAWTEvent(MouseEvent.MOUSE_CLICKED));
  }

  public void processStimulus (Enumeration criteria)
  {
    WakeupCriterion wakeup;
    AWTEvent[] event;
    int eventId;

    while (criteria.hasMoreElements())
    {
      wakeup = (WakeupCriterion) criteria.nextElement();
      if (wakeup instanceof WakeupOnAWTEvent)
      {
	event = ((WakeupOnAWTEvent)wakeup).getAWTEvent();
	for (int i=0; i<event.length; i++)
        {
	  eventId = event[i].getID();
	  if (eventId == (MouseEvent.MOUSE_CLICKED))
          {
	    final int x = ((MouseEvent)event[i]).getX();
	    final int y = ((MouseEvent)event[i]).getY();
	    pickCanvas.setShapeLocation(x, y);

	    final Point3d eyePos = pickCanvas.getStartPosition();
            pickResult = pickCanvas.pickClosest();

	    if (pickResult != null)
            {
              final SceneGraphPath pathToResult = pickResult.getSceneGraphPath();
              final Node checkNode = pathToResult.getNode(1);
              final Class checkClass = checkNode.getClass();

              if (checkClass.getName().endsWith("ES_BranchGroup"))
              {
                final ES_BranchGroup checkESbg = (ES_BranchGroup)checkNode;

                final ES_LifeForm myLife = checkESbg.myLife;

                if (myLife.hasFlag(ES.Bononaf))
                {
                  ES_Bononaf myBononaf = (ES_Bononaf)myLife;

                  myWindow.myInfoWindow.showInfoOn(myBononaf);
                }
                else if (myLife.hasFlag(ES.Desolok))
                {
                  ES_Desolok myDesolok = (ES_Desolok)myLife;

                  myWindow.myInfoWindow.showInfoOn(myDesolok);
                }
                else if (myLife.hasFlag(ES.BononafHive))
                {
                  ES_BononafHive myBononafHive = (ES_BononafHive)myLife;

                  myWindow.myInfoWindow.showInfoOn(myBononafHive);
                }
                else if (myLife.hasFlag(ES.BononafChild))
                {
                  ES_BononafChild myBononafChild = (ES_BononafChild)myLife;

                  myWindow.myInfoWindow.showInfoOn(myBononafChild);
                }
                else if (myLife.hasFlag(ES.Flower))
                {
                  ES_Flower myFlower = (ES_Flower)myLife;

                  myWindow.myInfoWindow.showInfoOn(myFlower);
                }
              }

            }
          }
        }
      }
    }
    wakeupOn (new WakeupOnAWTEvent(MouseEvent.MOUSE_CLICKED));
  }
}