/******************************************************************************

ES_ThinkBehavior
-------------------------------------------------------------------------------
The Basic thinking behavior that wakes up every n milliseconds

-------------------------------------------------------------------------------
Author:     Jochen Werner
Changed:    Feb 7, 2002
Version:    1.0

******************************************************************************/

package EcoSystem;

import javax.media.j3d.*;
import javax.vecmath.*;
import java.lang.Double;
import java.util.Enumeration;


public class ES_ThinkBehavior extends Behavior
{

  protected long cur_wakeupTime = 0;
  protected long repeats = -1;
  protected WakeupOnElapsedTime wakeupOnTime = null;
  protected ES_LifeForm myLife = null;

    public void initialize() {
	// Establish initial wakeup criteria
	wakeupOn(wakeupOnTime);
    }


    /**
     *  Override Behavior's stimulus method to handle the event.
     */
    public void processStimulus(Enumeration criteria)
    {
        myLife.Think();

        if (repeats != 0)
        {
          if (wakeupOnTime.getElapsedFrameTime() != cur_wakeupTime)
          {
            wakeupOnTime = null;
            wakeupOnTime = new WakeupOnElapsedTime(cur_wakeupTime);
          }
          wakeupOn(wakeupOnTime);

          if ( repeats > 0 )
            repeats--;
        }
    }

    public void setNextThink( long nextthink )
    {
        cur_wakeupTime = nextthink;
    }

    public long getNextThink()
    {
      return cur_wakeupTime;
    }

    public ES_ThinkBehavior(ES_LifeForm lf, long nextthink, long rep)
    {
        myLife = lf;
        cur_wakeupTime = nextthink;
        repeats = rep;
        wakeupOnTime = new WakeupOnElapsedTime(nextthink);

	this.setSchedulingBounds(ES_Util.GetInfiniteBounds());
    }
}




