/******************************************************************************

ES_Util
-------------------------------------------------------------------------------
Utility class that cannot be instantinated but holds useful static methods

-------------------------------------------------------------------------------
Author:     Jochen Werner
Changed:    Feb 7, 2002
Version:    1.0

******************************************************************************/

package EcoSystem;

import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.util.*;

import com.sun.j3d.loaders.*;
import java.io.File;

import java.awt.Component;
import java.io.IOException;
import com.sun.j3d.utils.image.TextureLoader;


public final class ES_Util {

 final protected static Random rnd = new Random();

 final public static Point3d zero = new Point3d(0,0,0);

 final public static Matrix3d zeroMatrix3d = new Matrix3d(0,0,0,0,0,0,0,0,0);

 public static long startTime = System.currentTimeMillis();

 final private static char aeiou[] = { 'a', 'e', 'i', 'o', 'u' };
 final private static char therest[] = { 'b', 'c', 'd', 'f', 'g', 'h', 'j', 'k', 'l', 'm', 'n', 'p', 'q', 'r', 's', 't', 'v', 'w', 'x', 'y', 'z' };

 private static ES_Window myWindow = null;

 private static float wd = Float.MAX_VALUE;


 public ES_Util() { }       // never instantiate this class

 public static void SetESWindow(ES_Window mywindow)
 {
   myWindow = mywindow;
 }

 public static void DebugPrintLn(String s)
 {
   System.out.println(s+(GetESTime()));
 }

 public static void    SetWorldSize(float f)
 {
   wd = f;
 }

 public static boolean IsInWorld(Point3f p)
 {
   if (p.x > 0)
     if (p.x > wd)
       return false;
   else
     if (p.x < -wd)
       return false;

   if (p.z > 0)
     if (p.z > wd)
       return false;
   else
     if (p.z < -wd)
       return false;

   return true;
 }

 public static boolean IsInWorld(Transform3D t)
 {
   final Point3f p = new Point3f();

   return IsInWorld(p);
 }

 public static boolean IsInWorld(ES_LifeForm lf)
 {
   return IsInWorld(lf.GetTransform());
 }

 public static long GetESTime()
 {
   return (System.currentTimeMillis() - startTime);
 }

 public static Bounds GetInfiniteBounds()
 {
   return new BoundingSphere(ES_Util.zero, Double.MAX_VALUE );
 }

 public static Transform3D CreateTansform3DFrom(Vector3d v)
 {
    final Transform3D retTrans = new Transform3D();
    retTrans.setTranslation(v);

    return retTrans;
 }

 public static void MakeLocalCoordinates(ES_LifeForm lf, Point3f worldcoordinates)
 {
   final Transform3D tmpTrans = lf.GetTransform();

   tmpTrans.invert();

   tmpTrans.transform(worldcoordinates);

 }
 public static boolean IsAnglePositive(Vector3d v1, Vector3d v2)
 {
    return (v2.x >= 0);
 }

 public static void PrintTrail(Group g, Point3d start, Point3d end)
 {
   final Appearance app = new Appearance();
   app.setColoringAttributes(new ColoringAttributes(0,0,0,ColoringAttributes.SHADE_GOURAUD));


   final LineArray myLine = new LineArray(2,LineArray.COORDINATES | LineArray.ALLOW_COORDINATE_WRITE );

   final Point3d[] points = {start,end};

   myLine.setCoordinates(0,points);
   myLine.setCapability(LineArray.ALLOW_INTERSECT);
   final Shape3D myShape = new Shape3D(myLine,app);
   ES_Util.SetESCapabilities(myShape);
   final BranchGroup bg = new BranchGroup();
   ES_Util.SetESCapabilities(bg);
   bg.addChild(myShape);
   g.addChild(bg);


 }

 public static void PrintCoordinateSystem(Group g, float scale)
 {
   final Appearance Xapp = new Appearance();
   Xapp.setColoringAttributes(new ColoringAttributes(255,0,0,ColoringAttributes.SHADE_GOURAUD));
   final Appearance Yapp = new Appearance();
   Yapp.setColoringAttributes(new ColoringAttributes(0,0,255,ColoringAttributes.SHADE_GOURAUD));
   final Appearance Zapp = new Appearance();
   Zapp.setColoringAttributes(new ColoringAttributes(0,255,0,ColoringAttributes.SHADE_GOURAUD));

   final Box         myXAxis   = new Box(1.0f*scale,0.01f,0.01f,Xapp);
   final Box         myYAxis   = new Box(0.01f,1.0f*scale,0.01f,Yapp);
   final Box         myZAxis   = new Box(0.01f,0.01f,1.0f*scale,Zapp);

   final TransformGroup Xsign = new TransformGroup(ES_Util.CreateTansform3DFrom(new Vector3d(-0.4,0,-0.2)));
   final Box         myXsign   = new Box(0.01f,0.01f,0.05f,Xapp);
   Xsign.addChild(myXsign);

   final TransformGroup Zsign = new TransformGroup(ES_Util.CreateTansform3DFrom(new Vector3d(-0.2,0,-0.4)));
   final Box         myZsign   = new Box(0.05f,0.01f,0.01f,Zapp);
   Zsign.addChild(myZsign);


   g.addChild(myXAxis);
   g.addChild(myYAxis);
   g.addChild(myZAxis);

   g.addChild(Xsign);
   g.addChild(Zsign);

 }

  public static boolean IsLifeFormInCyl(BranchGroup b, Transform3D origin, double cylradius, int LFflag)
  {
    final Point3d   cylstart =    new Point3d();
    origin.transform(cylstart);
    final Point3d   cylend   =    new Point3d(cylstart);

    cylend.y += 1;

    final PickCylinderSegment checkPick = new PickCylinderSegment(cylstart,cylend,cylradius);

    if (!b.isLive())
    {
      return false;
    }
    final SceneGraphPath checkResults[] = b.pickAll(checkPick);

    if (checkResults != null)
    {
      for (int i=0;i<checkResults.length;i++)
      {
        final Node checkNode = checkResults[i].getNode(1);

        final Class checkClass = checkNode.getClass();

        if (checkClass.getName().endsWith("ES_BranchGroup"))
        {
          final ES_BranchGroup checkESbg = (ES_BranchGroup)checkNode;
          if (checkESbg.myLife.hasFlag(LFflag))
          {
            //System.out.println("Other LifeForm found");
            return true;
          }
        }
      }
   }
//       System.out.println("No Other LifeForm found");
       return false;
  }

  public static Vector GetLifeFormsInCyl(BranchGroup b, Transform3D origin, double cylradius, int LFflag)
  {
    final Point3d   cylstart =    new Point3d();
    origin.transform(cylstart);
    final Point3d   cylend   =    new Point3d(cylstart);

    cylend.y += 1;

    final PickCylinderSegment checkPick = new PickCylinderSegment(cylstart,cylend,cylradius);

    if (!b.isLive())
    {
      return new Vector();
    }

    final Vector retVec = new Vector();
    final SceneGraphPath checkResults[] = b.pickAll(checkPick);

    if (checkResults != null)
    {
      for (int i=0;i<checkResults.length;i++)
      {
        final Node checkNode = checkResults[i].getNode(1);

        final Class checkClass = checkNode.getClass();

        if (checkClass.getName().endsWith("ES_BranchGroup"))
        {
          final ES_BranchGroup checkESbg = (ES_BranchGroup)checkNode;
          if (checkESbg.myLife.hasFlag(LFflag))
          {
            if (!retVec.contains(checkESbg.myLife))
              retVec.addElement(checkESbg.myLife);
          }
        }
      }
   }
       return retVec;
  }

  public static ES_LifeForm GetClosestLifeFormInCone(BranchGroup b, Transform3D origin, double coneLength, double coneAngle, int LFflag)
  {
    final Point3d   conestart =    new Point3d();
    final Point3d   coneend   =    new Point3d(0,0,coneLength);
    origin.transform(conestart);
    origin.transform(coneend);

    final PickConeSegment checkPick = new PickConeSegment(conestart,coneend,coneAngle);

    if (!b.isLive())
    {
      return null;
    }

    final SceneGraphPath checkResults[] = b.pickAllSorted(checkPick);

    if (checkResults != null)
    {
      for (int i=(checkResults.length-1);i>-1;i--)
      {
        final Node checkNode = checkResults[i].getNode(1);
        final Class checkClass = checkNode.getClass();

        if (checkClass.getName().endsWith("ES_BranchGroup"))
        {
          final ES_BranchGroup checkESbg = (ES_BranchGroup)checkNode;
          if (checkESbg.myLife.hasFlag(LFflag))
          {
            return checkESbg.myLife;
          }
        }
      }
   }
   return null;
  }

  public static ES_LifeForm GetClosestLifeFormInCone(BranchGroup b, Transform3D origin, double coneLength, double coneAngle, int LFflag, int LFflag2)
  {
    final Point3d   conestart =    new Point3d();
    final Point3d   coneend   =    new Point3d(0,0,coneLength);
    origin.transform(conestart);
    origin.transform(coneend);

    final PickConeSegment checkPick = new PickConeSegment(conestart,coneend,coneAngle);

    if (!b.isLive())
    {
      return null;
    }
    final SceneGraphPath checkResults[] = b.pickAllSorted(checkPick);

    if (checkResults != null)
    {
      for (int i=(checkResults.length-1);i>-1;i--)
      {
        final Node checkNode = checkResults[i].getNode(1);
        final Class checkClass = checkNode.getClass();

        if (checkClass.getName().endsWith("ES_BranchGroup"))
        {
          final ES_BranchGroup checkESbg = (ES_BranchGroup)checkNode;
          if (checkESbg.myLife.hasFlag(LFflag,LFflag2))
          {
            return checkESbg.myLife;
          }
        }
      }
   }
   return null;
  }

  public static ES_LifeForm GetClosestLifeFormInCone(BranchGroup b, Transform3D origin, double coneLength, double coneAngle, int LFflag, int LFflag2, int LFflag3)
  {
    final Point3d   conestart =    new Point3d();
    final Point3d   coneend   =    new Point3d(0,0,coneLength);
    origin.transform(conestart);
    origin.transform(coneend);

    final PickConeSegment checkPick = new PickConeSegment(conestart,coneend,coneAngle);

    if (!b.isLive())
    {
      return null;
    }
    final SceneGraphPath checkResults[] = b.pickAllSorted(checkPick);

    if (checkResults != null)
    {
      for (int i=(checkResults.length-1);i>-1;i--)
      {
        final Node checkNode = checkResults[i].getNode(1);
        final Class checkClass = checkNode.getClass();

        if (checkClass.getName().endsWith("ES_BranchGroup"))
        {
          final ES_BranchGroup checkESbg = (ES_BranchGroup)checkNode;
          if (checkESbg.myLife.hasFlag(LFflag,LFflag2,LFflag3))
          {
            return checkESbg.myLife;
          }
        }
      }
   }
   return null;
  }

  public static boolean CanSeeLifeFormInCone(BranchGroup b, Transform3D origin, double coneLength, double coneAngle, ES_LifeForm cansee)
  {
    final Point3d   conestart =    new Point3d();
    final Point3d   coneend   =    new Point3d(0,0,coneLength);
    origin.transform(conestart);
    origin.transform(coneend);

    final PickConeSegment checkPick = new PickConeSegment(conestart,coneend,coneAngle);

    if (!b.isLive())
    {
      return false;
    }
    final SceneGraphPath checkResults[] = b.pickAll(checkPick);

    if (checkResults != null)
    {
      for (int i=(checkResults.length-1);i>-1;i--)
      {
        final Node checkNode = checkResults[i].getNode(1);
        final Class checkClass = checkNode.getClass();

        if (checkClass.getName().endsWith("ES_BranchGroup"))
        {
          final ES_BranchGroup checkESbg = (ES_BranchGroup)checkNode;

          if (checkESbg.myLife.equals(cansee))
          {
            return true;
          }
        }
      }
   }
   return false;
  }

  public static void BuildRoute(ES_Animal animal, Transform3D goal, float offset, float movespeed, float lookspeed)
  {
    final Vector3f tmpvec = new Vector3f();
    goal.get(tmpvec);
    final Point3f  tmpPoint = new Point3f((Tuple3f)tmpvec);

    ES_Util.MakeLocalCoordinates(animal,tmpPoint);

    tmpvec.set((Tuple3f)tmpPoint);
    tmpvec.normalize();
    tmpvec.scale(offset);
    tmpPoint.sub((Tuple3f)tmpvec);
    animal.myMoves.AbortCurMove();
    animal.myMoves.ClearQueue();
    animal.myMoves.EnqueueMove(new ES_Move(animal,animal.myMoves,movespeed, lookspeed,tmpPoint));
    animal.myMoves.SetMoveMode(ES.MM_MoveASAP);
    animal.setFlag(ES.isMoving);

  }

  public static Appearance GetTextureApp(String texFile)
  {
    final Appearance retApp = new Appearance();

    final java.awt.Component observer = null;
    Texture texture = null;
    TextureLoader texLoader = null;

    //        if (this.getBasePath() != null)
//          texFile = this.getBasePath() + texFile;
    try
    {
      texLoader = new TextureLoader(texFile, observer);
      texture = texLoader.getTexture();
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }

    System.out.println((texture == null ? "Texture is null" : ""));

    retApp.setTexture(texture);

    Color3f white = new Color3f(1,1,1);
    Color3f black = new Color3f(0,0,0);
    final Material mat = new Material(white,white,white,white,0);
    mat.setLightingEnable(false);
    retApp.setMaterial(mat);

    final TextureAttributes ta = new TextureAttributes();
    ta.setTextureMode(TextureAttributes.MODULATE);
//    ta.setTextureBlendColor(0.0f,0.0f,0.0f,0.0f); // diffuse color
    retApp.setTextureAttributes(ta);

    final RenderingAttributes ra  = new RenderingAttributes();
    ra.setIgnoreVertexColors(true);
    retApp.setRenderingAttributes(ra);

    return retApp;
  }

 public static String GetRandomName()
 {
 final int strlen = (int)ES_Util.GetRndLong(3,8);
 boolean userest = false;

 final StringBuffer str = new StringBuffer();
 str.ensureCapacity(strlen+1);
 str.append((char)ES_Util.GetRndLong(65,90));

 for (int i=0; i<strlen; i++)
 {
   if (userest)
     str.append(therest[(int)ES_Util.GetRndLong(1,21)-1]);
   else
     str.append(aeiou[(int)ES_Util.GetRndLong(1,5)-1]);

   userest = !userest;
 }

 return new String(str);
 }

 public static float GetRndFloat()
 {
   return rnd.nextFloat();
 }

 public static float GetRndFloat(float start, float end)
 {
   return start + ((end-start) * rnd.nextFloat());
 }

 public static long GetRndLong()
 {
   return rnd.nextLong();
 }

 public static float GetRndLong(long start, long end)
 {
   return start + Math.round((end - start) * rnd.nextFloat());

 }

 public static boolean GetRndBool()
 {
   return rnd.nextBoolean();
 }

 public static boolean GetProbability(float percentage)
 {
   return (GetRndFloat(0,100) <= percentage);
 }

 public static void SetESCapabilities(BranchGroup b)
 {
   b.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
   b.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND );
   b.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE );
//   b.setCapability(BranchGroup.ALLOW_LOCAL_TO_VWORLD_READ );
   b.setCapability(BranchGroup.ALLOW_BOUNDS_READ );
   b.setCapability(BranchGroup.ALLOW_BOUNDS_WRITE );
//   b.setCapability(BranchGroup.ALLOW_AUTO_COMPUTE_BOUNDS_READ  );
//   b.setCapability(BranchGroup.ALLOW_AUTO_COMPUTE_BOUNDS_WRITE );
//   b.setCapability(BranchGroup.ALLOW_COLLIDABLE_READ );
//   b.setCapability(BranchGroup.ALLOW_COLLIDABLE_WRITE );
//   b.setCapability(BranchGroup.ALLOW_COLLISION_BOUNDS_READ );
//   b.setCapability(BranchGroup.ALLOW_COLLISION_BOUNDS_WRITE );
   b.setCapability(BranchGroup.ALLOW_DETACH );
   b.setCapability(BranchGroup.ALLOW_PICKABLE_READ );
   b.setCapability(BranchGroup.ALLOW_PICKABLE_WRITE );
//   b.setCapability(BranchGroup.ENABLE_COLLISION_REPORTING );
   b.setCapability(BranchGroup.ENABLE_PICK_REPORTING );
 }

 public static void SetESCapabilities(javax.media.j3d.TransformGroup t)
 {
  t.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
  t.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
  t.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
  t.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND );
  t.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE );
//  t.setCapability(TransformGroup.ALLOW_LOCAL_TO_VWORLD_READ );
  t.setCapability(TransformGroup.ALLOW_BOUNDS_READ );
  t.setCapability(TransformGroup.ALLOW_BOUNDS_WRITE );
//  t.setCapability(TransformGroup.ALLOW_AUTO_COMPUTE_BOUNDS_READ  );
//  t.setCapability(TransformGroup.ALLOW_AUTO_COMPUTE_BOUNDS_WRITE );
//  t.setCapability(TransformGroup.ALLOW_COLLIDABLE_READ );
//  t.setCapability(TransformGroup.ALLOW_COLLIDABLE_WRITE );
//  t.setCapability(TransformGroup.ALLOW_COLLISION_BOUNDS_READ );
//  t.setCapability(TransformGroup.ALLOW_COLLISION_BOUNDS_WRITE );
  t.setCapability(TransformGroup.ALLOW_PICKABLE_READ );
  t.setCapability(TransformGroup.ALLOW_PICKABLE_WRITE );
//  t.setCapability(TransformGroup.ENABLE_COLLISION_REPORTING );
  t.setCapability(TransformGroup.ENABLE_PICK_REPORTING );
 }

 public static void SetESCapabilities(Shape3D t)
 {
  t.setCapability(Shape3D.ALLOW_GEOMETRY_READ);
  t.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
  t.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
  t.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
  t.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND );
  t.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE );
//  t.setCapability(TransformGroup.ALLOW_LOCAL_TO_VWORLD_READ );
  t.setCapability(TransformGroup.ALLOW_BOUNDS_READ );
  t.setCapability(TransformGroup.ALLOW_BOUNDS_WRITE );
//  t.setCapability(TransformGroup.ALLOW_AUTO_COMPUTE_BOUNDS_READ  );
//  t.setCapability(TransformGroup.ALLOW_AUTO_COMPUTE_BOUNDS_WRITE );
//  t.setCapability(TransformGroup.ALLOW_COLLIDABLE_READ );
//  t.setCapability(TransformGroup.ALLOW_COLLIDABLE_WRITE );
//  t.setCapability(TransformGroup.ALLOW_COLLISION_BOUNDS_READ );
//  t.setCapability(TransformGroup.ALLOW_COLLISION_BOUNDS_WRITE );
  t.setCapability(TransformGroup.ALLOW_PICKABLE_READ );
  t.setCapability(TransformGroup.ALLOW_PICKABLE_WRITE );
//  t.setCapability(TransformGroup.ENABLE_COLLISION_REPORTING );
 }


 public static void InsertLifeForms(String lftype, int count, float circle)
 {
   final Transform3D moveTrans = new Transform3D();

   if (lftype.equalsIgnoreCase("Bononaf"))
   {
     for (int i=0; i<count; i++)
     {
       moveTrans.setTranslation(new Vector3d(GetRndFloat(-circle,circle),0,GetRndFloat(-circle,circle)));
       ES_Bononaf newBononaf = new ES_Bononaf(new Transform3D(moveTrans),myWindow);
     }
   }
   if (lftype.equalsIgnoreCase("Desolok"))
   {
     for (int i=0; i<count; i++)
     {
       moveTrans.setTranslation(new Vector3d(GetRndFloat(-circle,circle),0,GetRndFloat(-circle,circle)));
       ES_Desolok newDesolok = new ES_Desolok(new Transform3D(moveTrans),myWindow);
     }
   }
   if (lftype.equalsIgnoreCase("Flower"))
   {
     for (int i=0; i<count; i++)
     {
       moveTrans.setTranslation(new Vector3d(GetRndFloat(-circle,circle),0,GetRndFloat(-circle,circle)));
       ES_Flower newFlower = new ES_Flower(new Transform3D(moveTrans),myWindow);
     }
   }

 }

}
