/******************************************************************************

ES_Window
-------------------------------------------------------------------------------
The EcoSystem Window that sets up the render canvas and holds all informations
Every LifeForm has a reference to its window and hence can access some globals
-------------------------------------------------------------------------------
Author:     Jochen Werner
Changed:    Feb 4, 2002
Version:    1.0

******************************************************************************/

package EcoSystem;

import java.applet.Applet;
import java.awt.BorderLayout;
import java.awt.event.*;
import java.awt.GraphicsConfiguration;
import java.awt.Color;
import java.awt.image.*;
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.behaviors.vp.*;
import com.sun.j3d.utils.behaviors.keyboard.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.picking.behaviors.*;
import com.sun.j3d.utils.picking.*;

public class ES_Window extends Applet {


  public SimpleUniverse u = null;
  public BranchGroup objRoot = new BranchGroup();
  public BranchGroup staticBranch = new BranchGroup();
  public int numFlowers  = 0;
  public int numBononafs = 0;
  public int numDesoloks = 0;
  public MainFrame myMainFrame = null;
  public ES_LifeFormInfoWindow myInfoWindow = null;

  public boolean enableDescriptions = false;

  public ES_ModelCache  myModelCache = new ES_ModelCache();

    public void createSceneGraph() {

        ES_Util.SetESCapabilities(objRoot);

        myModelCache.addModel("./bononaf_male.ms3d", "BononafMale");
        myModelCache.addModel("./bononaf_female.ms3d", "BononafFemale");
        myModelCache.addModel("./bononaf_child.ms3d", "BononafChild");
        myModelCache.addModel("./bononaf_hive.ms3d", "BononafHive");
        myModelCache.addModel("./bononaf_male_corpse.ms3d", "BononafCorpseMale");
        myModelCache.addModel("./bononaf_female_corpse.ms3d", "BononafCorpseFemale");
        myModelCache.addModel("./bononaf_child_corpse.ms3d", "BononafCorpseChild");
        myModelCache.addModel("./desolok_male.ms3d", "DesolokMale");
        myModelCache.addModel("./desolok_female.ms3d", "DesolokFemale");
        myModelCache.addModel("./desolok_child.ms3d", "DesolokChild");
        myModelCache.addModel("./desolok_hive.ms3d", "DesolokHive");
        myModelCache.addModel("./desolok_male_corpse.ms3d", "DesolokCorpseMale");
        myModelCache.addModel("./desolok_female_corpse.ms3d", "DesolokCorpseFemale");
        myModelCache.addModel("./desolok_child_corpse.ms3d", "DesolokCorpseChild");
        myModelCache.addModel("./hipoly_plant.ms3d", "Plant");
        myModelCache.addModel("./plant_stalk.ms3d", "Plant_Stalk");
        myModelCache.addModel("./plant_blossom.ms3d", "Plant_Blossom");
        myModelCache.addModel("./plant_leaf.ms3d", "Plant_Leaf");
        myModelCache.addModel("./lp_plant_blossom.ms3d", "LP_Plant_Blossom");
        myModelCache.addModel("./lp_plant_leaf.ms3d", "LP_Plant_Leaf");
        myModelCache.addModel("./cloud.ms3d", "Cloud");
        myModelCache.addModel("./rain.ms3d", "Rain");

        Appearance  gndapp    = new Appearance(); //ES_Util.GetTextureApp("./grass.jpg");
        gndapp.setColoringAttributes(new ColoringAttributes(0.0f,0.4f,0.0f,ColoringAttributes.SHADE_GOURAUD));

        Box         myGround  = new Box(15.0f,0.0001f,15.0f,gndapp);

        staticBranch.addChild(myGround);

        ES_Util.PrintCoordinateSystem(staticBranch,15.0f);

        staticBranch.clearCapability(Node.ENABLE_PICK_REPORTING);

        Background myBack = new Background(new Color3f(0.4f,0.4f,1.0f));

        myBack.setApplicationBounds(ES_Util.GetInfiniteBounds());
        staticBranch.addChild(myBack);

    }

    public ES_Window()
    {
    }

    public void setMainFrame(MainFrame mf)
    {
      myMainFrame = mf;
    }

    public void init() {

      final long initstart = System.currentTimeMillis();

	setLayout(new BorderLayout());

        GraphicsConfiguration config = SimpleUniverse.getPreferredConfiguration();

	Canvas3D c = new Canvas3D(config);
        c.setDoubleBufferEnable(true);

        add("Center", c);

	createSceneGraph();
	u = new SimpleUniverse(c);

        u.getViewingPlatform().setNominalViewingTransform();

      //  OrbitBehavior myOrbit = new OrbitBehavior(c);
      //  myOrbit.setSchedulingBounds(ES_Util.GetInfiniteBounds() );

        ES_WalkBehavior myWalk = new ES_WalkBehavior(c,ES_WalkBehavior.KEY_LISTENER+ES_WalkBehavior.MOUSE_LISTENER+ES_WalkBehavior.MOUSE_MOTION_LISTENER,this);
        myWalk.setSchedulingBounds(ES_Util.GetInfiniteBounds() );

  //      u.getViewingPlatform().setViewPlatformBehavior( myOrbit );
        u.getViewingPlatform().setViewPlatformBehavior( myWalk );
        ES_PickMouseBehavior myMousePick = new ES_PickMouseBehavior(c,objRoot,this);
        myMousePick.setSchedulingBounds(ES_Util.GetInfiniteBounds());

        objRoot.addChild(myMousePick);

        myInfoWindow = new ES_LifeFormInfoWindow(this);

//        ES_Cloud myCloud = new ES_Cloud(ES_Util.CreateTansform3DFrom(new Vector3d(0,5,0)),this);

//        ExponentialFog myFog = new ExponentialFog(new Color3f(1.0f,1.0f,1.0f),1.0f);
//        myFog.setInfluencingBounds(ES_Util.GetInfiniteBounds());

//        objRoot.addChild(myFog);
        objRoot.compile();

        ES_Util.SetESWindow(this);
        ES_Util.SetWorldSize(15.0f);

	u.addBranchGraph(objRoot);
        u.addBranchGraph(staticBranch);

        ES_Util.InsertLifeForms("Flower" ,50,10.0f);
        ES_Util.InsertLifeForms("Bononaf",30,8.0f);
        ES_Util.InsertLifeForms("Desolok",5,8.0f);

        System.out.println("Init done in "+((System.currentTimeMillis()-initstart)/1000.0d)+" seconds.");

    }

    public void destroy() {
	u.removeAllLocales();
    }

    public static void main(String[] args) {
        ES_Window myWindow = new ES_Window();
        MainFrame myMainFrame = new MainFrame(myWindow, 400, 400);
        myMainFrame.setTitle("EcoSystem Initializing...");
        myWindow.setMainFrame(myMainFrame);
    }
}
